// CPPMixerSplitterUnitOperation.h : Declaration of the CCPPMixerSplitterUnitOperation

#pragma once
#include "resource.h"       // main symbols

#include "CPPMixerSplitterexample.h"
#include "CAPEOPENBaseObject.h"
#include "Collection.h"
#include "RealParameter.h"
#include "MaterialPort.h"
#include "EditDialog.h"

#define CURRENTFILEVERSIONNUMBER 0

//! Unit operation implementation class
/*!
  This is the main Unit Operation implementation class.
  Other than the standard interfaces (in CAPEOPENBaseObject)
  this object supplies ICapeUnit, ICapeUtilities and 
  IPersistStream implementations.
  
  This is the only object that will be created externally by 
  CAPE-OPEN simulation environments. Therefore, COM registration 
  of this object is required. The registry entries are in the 
  CPPMixerSplitterUnitOperation.rgs file; the rgs files have 
  been modified for registration in the user part of the 
  registry, rather than in HKCR. Additional registry entries
  for the Unit Operation includes the CATIDs for CAPE-OPEN
  objects and CAPE-OPEN Unit Operations, as well as the 
  CapeDescription key and its values.
  
*/


class ATL_NO_VTABLE CCPPMixerSplitterUnitOperation :
	public CComCoClass<CCPPMixerSplitterUnitOperation, &CLSID_CPPMixerSplitterUnitOperation>,
	public IDispatchImpl<ICapeUnit, &__uuidof(ICapeUnit), &LIBID_CAPEOPEN110, /* wMajor = */ 1, /* wMinor = */ 1>,
	public IDispatchImpl<ICapeUtilities, &__uuidof(ICapeUtilities), &LIBID_CAPEOPEN110, /* wMajor = */ 1, /* wMinor = */ 1>,
	public IPersistStream,
	public CAPEOPENBaseObject,
	public IDispatchImpl<ICapeUnitReport, &__uuidof(ICapeUnitReport), &LIBID_CAPEOPEN110, /* wMajor = */ 1, /* wMinor = */ 1>

{

public:

	//data members

	CapeValidationStatus valStatus; /*!< current valiation status */
	//CollectionObject *portCollection; /*!< the port collection */
	CollectionObject *parameterCollection; /*!< the parameter collection */
	IDispatch *simulationContext; /*!< reference to the simulation context, if any */
	int nCompounds; /*!< number of compounds; set at Validate(), used at Calculate() */
	int selectedReportIndex;

	//! Constructor
	/*!
	Calls the constructor of the CAPEOPENBaseObject base class with name and description initializers
	and initializes variables
	*/

	CCPPMixerSplitterUnitOperation() : 
	CAPEOPENBaseObject(true,
		L"CPP Mixer Splitter Example",
		L"Microsoft Visual C++ 2005 Mixer and Splitter Example according to CAPE-OPEN Unit Operation specification"
		)
	{//init variables
		valStatus=CAPE_NOT_VALIDATED;
		simulationContext=NULL;
		dirty=false;
		selectedReportIndex=-1;
		//create port collection
		portCollection=CCollection::CreateCollection(L"Port collection",L"Port collection for CPP Mixer Splitter");
		//create the ports
		MaterialPortObject *port;
		port=MaterialPortObject::CreateMaterialPort(L"Feed 1",L"Feed port for CPP Mixer Splitter Unit Operation example",CAPE_INLET);
		portCollection->AddItem(port); //item 0
		//port=MaterialPortObject::CreateMaterialPort(L"Feed 2",L"Feed port for CPP Mixer Splitter Unit Operation example",CAPE_INLET);
		//portCollection->AddItem(port); //item 1
		port=MaterialPortObject::CreateMaterialPort(L"Product 1",L"Product port for CPP Mixer Splitter Unit Operation example",CAPE_OUTLET);
		portCollection->AddItem(port); //item 2
		//port=MaterialPortObject::CreateMaterialPort(L"Product 2",L"Product port for CPP Mixer Splitter Unit Operation example",CAPE_OUTLET);
		//portCollection->AddItem(port); //item 3
		//create parameter collection
		parameterCollection=CCollection::CreateCollection(L"Parameter collection",L"Parameter collection for CPP Mixer Splitter");
		//create the parameters
		//RealParameterObject *par;
		//vector<double> dimensionality; //order: m, kg, S, A, K, mole, cd, rad, optionally followed by a delta indicator; trailing zeroes can be omitted
		//dimensionality.resize(0); //no dimension for this parameter
		//par=RealParameterObject::CreateParameter(L"Split factor",L"Split factor: fraction of product that goes to Product 1 stream",0,1,0.5,dimensionality,&valStatus); 
		//parameterCollection->AddItem(par); //parameter 0
		////the dimensionality of heat input is W = J / s = kg m ^2 / s ^3
		//dimensionality.resize(3);
		//dimensionality[0]=2.0; //m
		//dimensionality[1]=1.0; //kg
		//dimensionality[2]=-3.0; //s
		//double NaN=numeric_limits<double>::quiet_NaN();
		//par=RealParameterObject::CreateParameter(L"Heat input",L"Heat input: energy added to the total product",NaN,NaN,0,dimensionality,&valStatus); //no min or max value, we pass NaN
		//parameterCollection->AddItem(par); //parameter 1
	}

	//! Destructor
	/*!
	Cleans up variables and objects allocated by this unit operation
	*/

	~CCPPMixerSplitterUnitOperation()
	{	unsigned int i;
		//in case terminate has not been called, clean up references to external objects
		Terminate();
		//clean up ports
		for (i=0;i<portCollection->items.size();i++) ((ICapeIdentification*)portCollection->items[i])->Release(); //it does not matter which Release we use, but we have to cast to one of the implemented interfaces as the C++ compiler wants to know which Release we mean
		//clean up port collection
		portCollection->Release();
		//clean up parameters
		for (i=0;i<parameterCollection->items.size();i++) ((ICapeIdentification*)parameterCollection->items[i])->Release();
		//clean up parameter collection
		parameterCollection->Release();
	}

	void OnConnectNewStream()
	{
	}

	//! Registration entry points
	/*!
	The IDR_CPPMIXERSPLITTERUNITOPERATION resource ID maps to the CPPMixerSplitterUnitOperation.rgs resource in resource.h
	*/

	DECLARE_REGISTRY_RESOURCEID(IDR_CPPMIXERSPLITTERUNITOPERATION)

	//! COM Map
	/*!
	These macros tell the underlying ATL COM object how to map from QueryInterface calls interface pointers. The 
	interfaces implemented in the base class are mapped in the BASEMAP macro, and include ICapeIdentification as
	well as the error interfaces
	*/

	BEGIN_COM_MAP(CCPPMixerSplitterUnitOperation)
		COM_INTERFACE_ENTRY2(IDispatch, ICapeUnit)
		COM_INTERFACE_ENTRY(ICapeUnit)
		COM_INTERFACE_ENTRY(ICapeUtilities)
		COM_INTERFACE_ENTRY(IPersistStream)
		BASEMAP
		COM_INTERFACE_ENTRY(ICapeUnitReport)
	END_COM_MAP()


	DECLARE_PROTECT_FINAL_CONSTRUCT()

	// ICapeUnit Methods

	//! ICapeUnit::ports
	/*!
	Returns the port collection for this unit operation.
	\param ports [out, retval] receives the port collection, cannot be NULL
	*/

	STDMETHOD(get_ports)(LPDISPATCH * ports)
	{	if (!ports) return E_POINTER; //invalid pointer
		//instead of doing this, we can also call QueryInterface on the port collection
		*ports=(ICapeCollection*)portCollection;
		portCollection->AddRef(); //the caller must release this object
		//all done
		return NOERROR;
	}

	//! ICapeUnit::get_ValStatus
	/*!
	Returns the validation status.
	\param ValStatus [out, retval] receives the current validation status, cannot be NULL
	*/

	STDMETHOD(get_ValStatus)(CapeValidationStatus * ValStatus)
	{	if (!ValStatus) return E_POINTER; //invalid pointer
		*ValStatus=valStatus;
		return NOERROR;
	}

	//! ICapeUnit::Calculate
	/*!
	Calculate the unit operation. This is the function that performs the actual model calculation.
	\sa Calculate()
	*/

	STDMETHOD(Calculate)()
	{	
		return NOERROR;
	}

	//! ICapeUnit::Validate
	/*!
	Validate the unit operation. The simulation environment must ensure that a unit operation is in 
	a valid state before calling Calculate. The purpose of this function is two-fold: 
	-# to signal to the simulation environment that the unit is ready or not ready to be calculated, with a textual
	message describing the problem in case the unit is not ready
	-# to perform any tasks that can be length and do not need to be repeated at each calculation, 
	such as obtaining component constant values

	If the unit is not valid, this should be reflected in the isValid parameter, Validate should not 
	produce an error return just because the unit is not valid.

	Validate must also update the validation status.
	\param message [in, out] textual description of reason for not being valid, if isValid is VARIANT_FALSE
	\param isValid [out, retval]
	\sa Calculate(), get_ValStatus()
	*/

	STDMETHOD(Validate)(BSTR * message, VARIANT_BOOL * isValid)
	{	if ((!message)||(!isValid)) return E_POINTER; //invalid pointer
		//assume innocent, until proven guilty
		*isValid=VARIANT_TRUE;

		valStatus=(*isValid)?CAPE_VALID:CAPE_INVALID;
		return NOERROR;
	}

	// ICapeUtilities Methods

	//! ICapeUtilities::get_parameters
	/*!
	Returns the parameter collection for a unit operation, if any.
	\param parameters [out, retval] receives the parameters collection, cannot be NULL
	*/

	STDMETHOD(get_parameters)(LPDISPATCH * parameters)
	{	if (!parameters) return E_POINTER; //invalid pointer
		//instead of doing this, we can also call QueryInterface on the parameter collection
		*parameters=(ICapeCollection*)parameterCollection;
		parameterCollection->AddRef(); //the caller must release this object
		//all done
		return NOERROR;
	}

	//! ICapeUtilities::put_simulationContext
	/*!
	Specifies a pointer to the simulation context. The simulation context may perform certain tasks that the 
	Unit Operation wants to use, such a logging via ICapeDiagnostics. The Simulation Context is stored, but
	unused by this implementation.
	\param simContext [in] new simulation context. Can be NULL.
	*/

	STDMETHOD(put_simulationContext)(LPDISPATCH simContext)
	{	//release old simulation context
		if (simulationContext)
		   {simulationContext->Release();
			simulationContext=NULL;
		   }
		//set new one 
		simulationContext=simContext;
		if (simulationContext) simulationContext->AddRef(); //we could have also added a reference using QueryInterface
		return NOERROR;
	}

	//! ICapeUtilities::Initialize
	/*!
	Must be called by the simulation environment to initialize the unit operation. Initialize must be called
	after IPersistStream::Load or IPersistStreamInit::Load, if the unit is restored from persistence, and 
	after IPersistStreamInit::InitNew, if implemented. All other calls to the unit operation must be made
	after the Initialize call.
	\sa Terminate()
	*/

	STDMETHOD(Initialize)()
	{	//does nothing
		return NOERROR;
	}

	//! ICapeUtilities::Terminate
	/*!
	Terminate the unit operation. Clean up resources and release any references to objects that are 
	owned by the simulation environment, such as the simulation context and objects connected to 
	ports. Must be the last function called by the simulation environment (in case the object was
	succesfully initialized) before releasing the object
	\sa Initialize()
	*/

	STDMETHOD(Terminate)()
	{	unsigned int i;
		//release the simulation context, if any
		if (simulationContext) 
		   {simulationContext->Release();
			simulationContext=NULL;
		   }
		//disconnect the ports
		for (i=0;i<portCollection->items.size();i++)
		   {MaterialPortObject *port=(MaterialPortObject *)portCollection->items[i]; //item is stored as CAPEOPENBaseObject, cast to port
			port->Disconnect();
		   }
		return NOERROR;
	}

	//! ICapeUtilities::Edit
	/*!
	Unit Operations can be configured via their parameters as exposed from the parameter collection.
	Edit provides another way for configuration of Unit Operations. Edit is typically called in 
	response to the user's request to configure the Unit Operation
	*/

	STDMETHOD(Edit)()
	{	
		MessageBox(NULL,_T("No Edit Dialog!"),_T("Tip"),MB_YESNO);
		return NOERROR;
	}

	//IPersistStream methods

	//! IPersistStream::IsDirty
	/*!
	Check whether we need to be saved
	\return S_OK if we need to be saved, or S_FALSE
	*/

	STDMETHOD(IsDirty)()
	{//we are dirty if the parameters are dirty as well
		unsigned int i;
		for (i=0;i<parameterCollection->items.size();i++)
		   {RealParameterObject *par=(RealParameterObject *)parameterCollection->items[i];
			if (par->dirty) 
			   {dirty=true;
				break;
			   }
		   }
		return (dirty)?S_OK:S_FALSE;
	}

	//! IPersistStream::Load
	/*!
	Restore from persistence
	\param pstm [in] IStream to load from
	\return S_OK for success, or S_FALSE
	\sa Save()
	*/

	STDMETHOD(Load)(IStream * pstm)
	{   if (!pstm) return E_POINTER;
		return S_OK;
	}

	//! IPersistStream::Save
	/*!
	Save to persistence
	\param pstm [in] IStream to save to
	\param fClearDirty [in] if set, we must clear the dirty flags
	\return S_OK for success, or S_FALSE
	\sa Load(), GetSizeMax()
	*/

	STDMETHOD(Save)(IStream * pstm, BOOL fClearDirty)
	{   if (!pstm) return E_POINTER;
		return S_OK;
	}

	//! IPersistStream::GetSizeMax
	/*!
	Return the maximum size required to save this object
	\param pcbSize [out] receives the size, cannot be NULL
	\sa Save()
	*/

	STDMETHOD(GetSizeMax)(_ULARGE_INTEGER * pcbSize)
	{   if (!pcbSize) return E_POINTER;
		pcbSize->QuadPart=0;
		return NOERROR;
	}

	//! IPersistStream::GetClassID
	/*!
	Return the CLSID of this object
	\param pClassID [out] receives the CLSID, cannot be NULL
	*/

	STDMETHOD(GetClassID)(CLSID *pClassID)
	{   if (!pClassID) return E_POINTER;
		*pClassID=CLSID_CPPMixerSplitterUnitOperation;
		return NOERROR;
	}


	// ICapeUnitReport Methods
	//  this is an optional interface; a sample report is implemented to show how it is done

	//! ICapeUnitReport::get_reports
	/*!
	Return the list of reports
	\param reports [out, retval] receives the list of reports, cannot be NULL
	*/

	STDMETHOD(get_reports)(VARIANT * reports)
	{	if (!reports) return E_POINTER; //not a valid pointer
	    CVariant reportList;
	    reportList.MakeArray(1,VT_BSTR);
	    reportList.AllocStringAt(0,L"Sample report");
	    *reports=reportList.ReturnValue(); //will be freed by caller, so make sure we do not own this value
		return NOERROR;
	}

	//! ICapeUnitReport::get_selectedReport
	/*!
	Return the selected report
	\param report [out, retval] receives the name of the selected report, cannot be NULL
	*/

	STDMETHOD(get_selectedReport)(BSTR * report)
	{	if (!report) return E_POINTER; //not a valid pointer
	    if (selectedReportIndex<0)
	     {SetError(L"A report was not selected",L"ICapeUnitReport",L"get_selectedReport");
	      return ECapeUnknownHR;
	     }
	    //the selected report must be the sample report
	    *report=SysAllocString(L"Sample report"); //will be freed by caller
		return NOERROR;
	}

	//! ICapeUnitReport::put_selectedReport
	/*!
	Select a report
	\param report [in] report to select, must be in the list returned by get_reports
	\sa get_reports()
	*/

	STDMETHOD(put_selectedReport)(BSTR report)
	{	//request to select report
	    if (CBSTR::Same(report,L"Sample report"))
	     {selectedReportIndex=0; 
	      return NOERROR;
	     }
	    //report not supported
	    SetError(L"Invalid report selection: no such report",L"ICapeUnitReport",L"put_selectedReport");
		return ECapeUnknownHR;
	}
	
	//! ICapeUnitReport::put_selectedReport
	/*!
	Return the content of the selected report
	\param reportContent [in, out] receives the content of the report; cannot be NULL
	*/
	
	STDMETHOD(ProduceReport)(BSTR * reportContent)
	{	if (!reportContent) return E_POINTER; //not a valid pointer
	    //this is [in, out]; this means we have to free what is currently in there
	    if (*reportContent) 
	     {SysFreeString(*reportContent);
	      *reportContent=NULL;
	     }
	    //did we select a report?
	    if (selectedReportIndex<0)
	     {SetError(L"A report was not selected",L"ICapeUnitReport",L"ProduceReport");
	      return ECapeUnknownHR;
	     }
	    //the selected report must be the sample report, return its content
	    *reportContent=SysAllocString(L"Example Mixer Splitter Report Content"); //caller must free this value
		return NOERROR;
	}
	
};

OBJECT_ENTRY_AUTO(__uuidof(CPPMixerSplitterUnitOperation), CCPPMixerSplitterUnitOperation)
